<?
/***************************************************************************
 *   Copyright (C) 2005 by J Randolph Smith                                *
 *   snortsms@servangle.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
include_once("func_Log.php");

/*************************************************/
/*** SNORT Rule Disection and Generation Class ***/
/*************************************************/

class Rule
	{
	var $raw;
	var $rulearray;
	var $direction;
	var $error;				// Place to put errors
	var $hash_header;		// hash used to reference sections of the rule header.
	var $payload_keywords;	// A hash of known rule option payload-related keywords

	function Rule ()
		{
		$this->rulearray = array();
		//$rulearray['header'] = array();
		//$rulearray['nonpayload'] = array();
		$this->direction = array("->","<>");
		$this->error = array();
		/* Define the header lookup table */
		$this->hash_header['action'] = 0;
		$this->hash_header['protocol'] = 1;
		$this->hash_header['src_ip'] = 2;
		$this->hash_header['src_port'] = 3;
		$this->hash_header['flow'] = 4;
		$this->hash_header['dst_ip'] = 5;
		$this->hash_header['dst_port'] = 6;		
		/* Define rule option payload keywords [ keyword => true/false (has parameters) ]*/
		$this->payload_keywords = array(	'content'		=>TRUE,
											'uricontent'	=>TRUE,
											'pcre'			=>TRUE,
											'regex'			=>TRUE,
											'byte_jump'		=>TRUE,
											'byte_test'		=>TRUE,	
											
											'depth'			=>TRUE,
											'distance'		=>TRUE,
											'ftpbounce'		=>FALSE,
											'isdataat'		=>TRUE,
											'nocase'		=>FALSE,
											'offset'		=>TRUE,
											'rawbytes'		=>FALSE,
											'within'		=>TRUE 		);

		
		}

	function Set ($varname,$value) 	{ $this->$varname=$value; }
	function Get ($varname) 		{ return ($this->$varname); }


	
	/*** Rule Parser Wrapper Method ***/
	function Parse($rule)
		{
		$this->_ParseRule($rule,$this->raw,$this->error,$this->rulearray);
		/*** Was rule disection a success ***/
		return ($this->_Validate());
		}
		
	/*** Actual Rule Parser ***/
	function _ParseRule(&$rule,&$raw,&$error,&$rulearray)
		{
		$raw 			= $rule;
		$rulearray 		= array();
		// Reset counters to an error state.
		$payload_cnt 	= -1;
		$nonpayload_cnt = -1;
		$metadata_cnt 	= -1;
		$postdetect_cnt = -1;
		
		/*** Get the Header and Option***/
		//preg_match("/(.*)\((.*)\)/", $rule, $matches);
		if ( ! preg_match("/([^\(]+)\((.*)\)/", $rule, $matches) )
			{
			array_push($error,"Rule Header or Options parsing failed");
			LogEvent("Error: " . $error);
			return false;
			}
		$rule_header = $matches[1];
		$rule_options = $matches[2];
		/*** Get all the Header values ****/	
		if ( ! preg_match_all("/([^\s]+)\s+?/", $rule_header, $header) )
			{
			array_push($error,"Rule Header parsing failed");
			LogEvent("Error: " . $error);
			return false;
			}
		$rulearray['header'] = $header[0];
		/*** Get all the Option tokens ****/
		/* we need to escape special chars 1st */
		$rule_options = preg_replace('/\\\\;/','__ESC_SEMICOLON__',$rule_options); /* preserve any '\;' as a token */
		if ( ! preg_match_all("/(\w+):?(\"[^\"]+\"|[^;]+)?;\s?/", $rule_options, $option) )
			{
			array_push($error,"Rule Options parsing failed");
			LogEvent("Error: " . $error);
			return false;
			}
		$count = count($option[1]);
		
		/* Push each option found onto arrays based on option categories */
		/* Group 'content' options and their modifiers together */
		for($i=0;$i<$count;$i++)
			{
		    $option[2][$i] = preg_replace('/__ESC_SEMICOLON__/','\\;',$option[2][$i]); /* convert tokens back to '\;' */
			$keyword = strtolower($option[1][$i]);
			switch($keyword)
				{
				case "content":
				case "uricontent":
				case "pcre":
				case "regex":	// Legacy support
				case "byte_test":
				case "byte_jump":
					/* For any of the keywords above we increment to keep 
					content-modifier options alined with their respective 
					content option */
					$payload_cnt++;	
				case "depth":
				case "offset":
				case "distance":
				case "within":
				case "isdataat":
					$rulearray['payload'][$payload_cnt][$keyword] = $option[2][$i];
					break;
				
				case "nocase":
				case "rawbytes":
				case "ftpbounce":
					$rulearray['payload'][$payload_cnt][$keyword] = true;
					break;

				/* Meta options get their own array */
				case "msg":
				case "sid":
				case "rev":
				case "classtype":
				case "priority":
				case "reference":
					$metadata_cnt++;
					$rulearray['metadata'][$metadata_cnt]['key'] = $keyword;
					$rulearray['metadata'][$metadata_cnt]['val'] = $option[2][$i];
					break;

				/* Post Detection options get their own array */
				case "logto":
				case "session":
				case "resp":
				case "react":
				case "tag":
				case "threshold":	// might need it's own section ??
				case "suppress":	// might need it's own section ??
					$postdetect_cnt++;
					$rulearray['postdetect'][$postdetect_cnt]['key'] = $keyword;
					$rulearray['postdetect'][$postdetect_cnt]['val'] = $option[2][$i];
					break;

				/* Everything else is just another non-payload option */
				default:
					$nonpayload_cnt++;
					$rulearray['nonpayload'][$nonpayload_cnt]['key'] = $keyword;
					$rulearray['nonpayload'][$nonpayload_cnt]['val'] = $option[2][$i];
					break;
				}
			
			}

			
			

			
		}


	/*********** 
	Returns the rule as a plain text string from the internal
	array structures.
	
	To help optimize the rule definition, we render each
	option section in a specific order.
	***********/
	function GenRule()
		{	
		$gen_rule = "";
		$count = count($this->rulearray['header']);
		for($i=0;$i<$count;$i++)
			{	
			$gen_rule .= $this->rulearray['header'][$i];			
			}
		$gen_rule .= "(";
			
		/*** Lets pump out the nonpayload options 1st 
		since some are very important and help make 
		the rule definition more efficient ***/
		$count = count($this->rulearray['nonpayload']);
		for($i=0;$i<$count;$i++)
			{	
			if ($this->rulearray['nonpayload'][$i]['val'] != "")
				{
				$gen_rule .= $this->rulearray['nonpayload'][$i]['key'] . ":" . $this->rulearray['nonpayload'][$i]['val'] . "; ";
				}
			else	
				{
				$gen_rule .= $this->rulearray['nonpayload'][$i]['key'] . "; ";
				}
			}
			
		/*** Next are pattern search related options ***/
		if( isset($this->rulearray['payload']) )
			{
			$count = count($this->rulearray['payload']);
			/* loop thru each payload option group */
			for($i=0;$i<$count;$i++)
				{				
				/* loop thru each option keyword and render the ones we find for this group  */
				foreach($this->payload_keywords as $kw=>$has_parameters)
					{
					if ( isset($this->rulearray['payload'][$i][$kw]) )
						{
						if( $has_parameters )
							{						
							$gen_rule .= $kw . ":" . $this->rulearray['payload'][$i][$kw] . "; ";
							}
						else
							{
							$gen_rule .= $kw . "; ";
							}
						}
					}
				}
			}


		/*** Lets render any Post-Detection options ***/
		if( isset($this->rulearray['postdetect']) )
			{
			$count = count($this->rulearray['postdetect']);
			for($i=0;$i<$count;$i++)
				{
				$gen_rule .= $this->rulearray['postdetect'][$i]['key'] . ":" . $this->rulearray['postdetect'][$i]['val'] . "; ";				
				}
			}
			
			
		/*** Finally lets render any meta data options ***/
		if( isset($this->rulearray['metadata']) )
			{
			$count = count($this->rulearray['metadata']);
			for($i=0;$i<$count;$i++)
				{
				$gen_rule .= $this->rulearray['metadata'][$i]['key'] . ":" . $this->rulearray['metadata'][$i]['val'] . "; ";				
				}
			}

		$gen_rule .= ")";
		return ($gen_rule);
		}


	function GetHeaderItem($section)
		{
		/* Lookup the header name to numeric index value */		
		if(isset($this->hash_header[$section]))
			{
			$index = $this->hash_header[$section];
			$return = $this->rulearray['header'][$index];
			}
		else
			{
			$return = "ERROR: Unknown header field passed in [$section]";
			}
		return($return);
		}



	/****************
	Returns the value of a requested option.
	- returns an array if more than one instance was found.
	- returns string if one instance was found.
	- returns NULL if not found.	
	****************/
	function getOption($section,$keyword)
		{
		$return = NULL;
		
		if(isset($this->rulearray[$section]))
			{
			$return = array();
			foreach( $this->rulearray[$section] as $item )
				{
				if($item['key'] == $keyword)
					{
					$value = $item['val'];
					array_push($return, $value );
					}					
				}

			if( count($return) == 1)
				{
				$return = $return[0];
				}
			elseif( count($return) < 1)
				{
				$return = NULL;
				}
			// else, do nothing, just return the array.
			
			}
		return($return);
		}


	/****************
	Modifies an existing option's value or creates a new option.
	!! Note, does not handle multiple instances yet - use with caution!
	****************/
	function setOption($section,$keyword,$value)
		{
		$found = false;

		switch($section)
			{
			case"metadata":
			case"nonpayload":
				$total = count($this->rulearray[$section]);
				$i=0;
				while($i<$total)	// see if option already exists, if so, update it.
					{
					if( $this->rulearray[$section][$i]['key'] == $keyword )
						{
						$this->rulearray[$section][$i]['val'] = $value;
						$found = true;
						$i = $total;
						}
					$i++;
					}
				if($found == false)	// If not found, then add as a new option.
					{
					$this->rulearray[$section][$i]['key'] = $keyword;
					$this->rulearray[$section][$i]['val'] = $value;
					}			
			break;		
			
			case"payload":
				$total = count($this->rulearray[$section]);
				$i=0;
				while($i<$total)	// see if option already exists, if so, update it.
					{
					if( isset($this->rulearray[$section][$i][$keyword]) )
						{
						$this->rulearray[$section][$i][$keyword] = $value;
						$found = true;
						$i = $total;
						}
					$i++;
					}
				if($found == false)	// If not found, then add as a new option.
					{
					$this->rulearray[$section][$i][$keyword] = $value;
					}			
			break;	
			
			
			}

		}



	function _Validate()
		{
		$this->error = array();
		$header_count	= count($this->rulearray['header']);
		$npl_options	= (is_array($this->rulearray['nonpayload']) ? true : false);
		$pl_options		= (is_array($this->rulearray['payload']) ? true : false);
		$pd_options		= (is_array($this->rulearray['postdetect']) ? true : false);
		$md_options		= (is_array($this->rulearray['metadata']) ? true : false);
		$msg			= $this->getOption("metadata","msg");
		if ($header_count <> 7)
			{
			array_push($this->error,"Rule Header Error");
			}
		if ( (!$npl_options) && (!$pl_options) && (!$pd_options) && (!$md_options))
			{
			array_push($this->error,"Rule Options Error");
			}
		if ($msg == "")
			{
			array_push($this->error,"Missing or invalid 'msg:' meta-data rule option.");
			}
			
		if( isset($this->rulearray['payload']) )
			{	
			if( is_array($this->rulearray['payload']) )
				{
				$key = key($this->rulearray['payload']);
				if( $key <= -1 )
					{
					array_push($this->error,"Stray 'content' modifier");
					}
				}
			}

		/* Were there any issues ? */
		if(count($this->error) > 0)
			{
			foreach($this->error as $error_msg)
				{
				LogEvent("Error: rule validation: " . $error_msg);
				}
			return false;
			}
		else
			{
			return true;
			}
		}



	/*********** Returns all the Payload related options as a single text string **********/
	function GetPayloadLines()
		{	
		$gen_string = "";
		$break = "";
		$bull = "; ";

		/*** Loop content ***/
		if( isset($this->rulearray['payload']) )
			{
			$count = count($this->rulearray['payload']);
			for($i=0;$i<$count;$i++)
				{
				/* loop thru each option keyword and render the ones we find for this group  */
				foreach($this->payload_keywords as $kw=>$has_parameters)
					{
					if ( isset($this->rulearray['payload'][$i][$kw]) )
						{
						if( $has_parameters )
							{						
							$gen_string .= $kw . ":" . $this->rulearray['payload'][$i][$kw] . $bull;
							}
						else
							{
							$gen_string .= $kw . $bull;
							}
						}
					}
				$gen_string .= "\n";
				}
			}
		return ($gen_string);
		}
		

	/*********** Returns all the non-payload related options as a single text string 	**********/
	function GetNonPayloadLines()
		{	
		$gen_string = "";
		/*** Loop options ***/	
		$count = count($this->rulearray['nonpayload']);
		for($i=0;$i<$count;$i++)
			{	
			if ($this->rulearray['nonpayload'][$i]['val'] != "")
				{
				$gen_string .= $break.$this->rulearray['nonpayload'][$i]['key'] . ":" . $this->rulearray['nonpayload'][$i]['val'] . "; ";
				}
			else	
				{
				$gen_string .= $break.$this->rulearray['nonpayload'][$i]['key'] . "; ";
				}
			$gen_string .= "\n";	
			}
		return ($gen_string);
		}
		
	
	/*********** Returns all the Post-detect 'options' as a text string 	**********/
	function GetPostDetectLines()
		{	
		$gen_string = "";
		$bull = ";\n";

		/*** Loop options ***/
		if( isset($this->rulearray['postdetect']) )
			{
			$count = count($this->rulearray['postdetect']);
			for($i=0;$i<$count;$i++)
				{
				$gen_string .= $break . $this->rulearray['postdetect'][$i]['key'] . ":" . $this->rulearray['postdetect'][$i]['val'] . $bull;
				}
			}
		return ($gen_string);
		}
		
	/*********** Returns all the meta data 'options' as a text string 	**********/
	function GetMetaDataLines()
		{	
		$gen_string = "";
		$break = "";
		$bull = ";\n";

		/*** Loop options ***/
		if( isset($this->rulearray['metadata']) )
			{
			$count = count($this->rulearray['metadata']);
			for($i=0;$i<$count;$i++)
				{
				$gen_string .= $break . $this->rulearray['metadata'][$i]['key'] . ":" . $this->rulearray['metadata'][$i]['val'] . $bull;
				}
			}
		return ($gen_string);
		}

		

	function ShowTree()
		{
		?>
		<p>
		<pre><b>*** DEBUG ***</b>	
		<?
		print_r($this->rulearray);
		?>
		<b>*************</b>
		</pre>
		</p>
		<?
		}






	/*************************/
	/**!!! not yet used !!!***/
	/*************************/
	function diffcontent(&$left,&$right,&$new)
		{
		$temp_ary = array_merge($left,$right); 
		foreach($temp_ary as $key=>$value)
			{
			if($right[$key] == $left[$key])
				{
				$val = $temp_ary[$key];				
				}
			elseif($right[$key] == "")
				{
				$val = "<font color=\"Gray\">" .$temp_ary[$key] . "</font>";			
				}
			elseif($left[$key] == "")
				{
				$val = "<font color=\"Cyan\">" .$temp_ary[$key] . "</font>";			
				}
			else
				{
				$val = "<font color=\"Red\">" .$temp_ary[$key] . "</font>";			
				}				
			$new[$key] = $val;
			}
		}
	
	/*************************/
	/**!!! not yet used !!!***/
	/*************************/
	function Differences($rule2)
		{
		$rule2_raw 		= "";
		$rule2_error 	= array();
		$rule2_rulearray = "";
		
		$this->_ParseRule($rule2,$rule2_raw,$rule2_error,$rule2_rulearray);

		//print"<PRE>\n";
		//var_dump($this->rulearray);
		//print"<HR>\n";
		//var_dump($rule2_rulearray);
		//print"</PRE>\n";

		$new_rulearray = "";
		
		print"<h3>Diffs</h3><HR>\n";

		/* Compare headers */
		$count = count($this->rulearray['header']);
		for($i=0;$i<$count;$i++)
			{
			if(  $this->rulearray['header'][$i] != $rule2_rulearray['header'][$i]  )
				{
				$new_rulearray['header'][$i] = "<font color=\"Red\">" . $rule2_rulearray['header'][$i] . "</font>";
				}
			else
				{
				$new_rulearray['header'][$i] = $rule2_rulearray['header'][$i];
				}
			}

		/* Compare references */
		/* rewrite needed */
		/*
		$count_L = count($this->rulearray['metadata']);
		$count_R = count($rule2_rulearray['metadata']);		
		$max_count = (  $count_L >=  $count_R ?  $count_L  :  $count_R  );		
		$i = 0;
		while($i < $max_count)
			{
			if(  ($this->rulearray['metadata'][$i] != "") && ($rule2_rulearray['metadata'][$i] == "")  )
				{
				$new_rulearray['metadata'][$i] = "<font color=\"Grey\">" . $this->rulearray['metadata'][$i] . "</font>";
				}
			elseif(  ($this->rulearray['metadata'][$i] == "") && ($rule2_rulearray['metadata'][$i] != "")  )
				{
				$new_rulearray['metadata'][$i] = "<font color=\"Cyan\">" . $rule2_rulearray['metadata'][$i] . "</font>";
				}
			elseif(  $this->rulearray['metadata'][$i] == $rule2_rulearray['metadata'][$i]  )
				{
				$new_rulearray['metadata'][$i] = $this->rulearray['metadata'][$i];
				}
			elseif(  $this->rulearray['metadata'][$i] != $rule2_rulearray['metadata'][$i]  )
				{
				$new_rulearray['metadata'][$i] = "<font color=\"Red\">" . $rule2_rulearray['metadata'][$i] . "</font>";
				}			
			$i++; 
			}
		*/

		/* Compare Content */
		$count_L = count($this->rulearray['payload']);
		$count_R = count($rule2_rulearray['payload']);	
		$max_count = (  $count_L >=  $count_R ?  $count_L  :  $count_R  );		
		$i = 0;
		while($i < $max_count)
			{
			$new_rulearray['payload'][$i] = array();
			$this->diffcontent($this->rulearray['payload'][$i],$rule2_rulearray['payload'][$i],$new_rulearray['payload'][$i]);

			$i++;
			}

		/* Compare options */
		$left_op = array();
		foreach($this->rulearray['nonpayload'] as $option)
			{
			$left_op[$option['key']] = $option['val'];
			}
		//print"<PRE>\n";
		//var_dump($left_op);
		//print"</PRE>\n";
		$right_op = array();
		foreach($rule2_rulearray['nonpayload'] as $option)
			{
			$right_op[$option['key']] = $option['val'];
			}
		//print"<PRE>\n";
		//var_dump($right_op);
		//print"</PRE>\n";
		$i =0;
		$temp_ary = array_merge($left_op,$right_op); 
		foreach($temp_ary as $key=>$value)
			{
			if($right_op[$key] == $left_op[$key])
				{
				$val = $temp_ary[$key];				
				}
			elseif($right_op[$key] == "")
				{
				$val = "<font color=\"Gray\">" .$temp_ary[$key] . "</font>";			
				}
			elseif($left_op[$key] == "")
				{
				$val = "<font color=\"Cyan\">" .$temp_ary[$key] . "</font>";			
				}
			else
				{
				$val = "<font color=\"Red\">" .$temp_ary[$key] . "</font>";			
				}				
			$new_rulearray['nonpayload'][$i]['key'] = $key;
			$new_rulearray['nonpayload'][$i]['val'] = $val;
			$i++;
			}
		//print"<PRE>\n";
		//var_dump($merge_op);
		//print"</PRE>\n";

		//print"<PRE>\n";
		//var_dump($new_rulearray);
		//print"</PRE>\n";
		}







	}	// *** END: Class
	
	?>
